@@ -23,23 +23,31 @@
 #ifndef __GST_ALSASRC_H__
 #define __GST_ALSASRC_H__
 
-
 #include <gst/audio/gstaudiosrc.h>
 #include "gstalsa.h"
 #include "gstalsamixer.h"
 
-
 G_BEGIN_DECLS
 
-#define GST_TYPE_ALSA_SRC          (gst_alsasrc_get_type())
-#define GST_ALSA_SRC(obj)          (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_ALSA_SRC,GstAlsaSrc))
-#define GST_ALSA_SRC_CLASS(klass)  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_ALSA_SRC,GstAlsaSrcClass))
-#define GST_IS_ALSA_SRC(obj)       (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ALSA_SRC))
-#define GST_IS_ALSA_SRC_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_ALSA_SRC))
+#define GST_TYPE_ALSA_SRC            (gst_alsasrc_get_type())
+#define GST_ALSA_SRC(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_ALSA_SRC,GstAlsaSrc))
+#define GST_ALSA_SRC_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_ALSA_SRC,GstAlsaSrcClass))
+#define GST_IS_ALSA_SRC(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ALSA_SRC))
+#define GST_IS_ALSA_SRC_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_ALSA_SRC))
+#define GST_ALSA_SRC_CAST(obj)       ((GstAlsaSrc *)(obj))
+
+#define GST_ALSA_SRC_GET_LOCK(obj)  (GST_ALSA_SRC_CAST (obj)->alsa_lock)
+#define GST_ALSA_SRC_LOCK(obj)      (g_mutex_lock (GST_ALSA_SRC_GET_LOCK (obj)))
+#define GST_ALSA_SRC_UNLOCK(obj)    (g_mutex_unlock (GST_ALSA_SRC_GET_LOCK (obj)))
 
 typedef struct _GstAlsaSrc GstAlsaSrc;
 typedef struct _GstAlsaSrcClass GstAlsaSrcClass;
 
+/**
+ * GstAlsaSrc:
+ *
+ * Opaque data structure
+ */
 struct _GstAlsaSrc {
   GstAudioSrc           src;
 
@@ -49,6 +57,8 @@ struct _GstAlsaSrc {
   snd_pcm_hw_params_t   *hwparams;
   snd_pcm_sw_params_t   *swparams;
 
+  GstCaps               *cached_caps;
+
   snd_pcm_access_t      access;
   snd_pcm_format_t      format;
   guint                 rate;
@@ -61,6 +71,8 @@ struct _GstAlsaSrc {
   snd_pcm_uframes_t     period_size;
 
   GstAlsaMixer          *mixer;
+
+  GMutex                *alsa_lock;
 };
 
 struct _GstAlsaSrcClass {
