@@ -34,22 +34,19 @@
 
 /**
  * SECTION:gstappsrc
- * @short_description: Easy way for applications to inject buffers into a
- *     pipeline
  * @see_also: #GstBaseSrc, appsink
  *
  * The appsrc element can be used by applications to insert data into a
  * GStreamer pipeline. Unlike most GStreamer elements, Appsrc provides
  * external API functions.
  *
- * appsrc can be used by linking with the libgstapp library to access the
- * methods directly or by using the appsrc action signals.
+ * appsrc can be used by linking to the gstappsrc.h header file to access the
+ * methods or by using the appsrc action signals. For the API
+ * documentation, see the documentation for libgstapp in the
+ * GStreamer Base Plugins Library reference...
  *
  * Before operating appsrc, the caps property must be set to a fixed caps
- * describing the format of the data that will be pushed with appsrc. An
- * exception to this is when pushing buffers with unknown caps, in which case no
- * caps should be set. This is typically true of file-like sources that push raw
- * byte buffers.
+ * describing the format of the data that will be pushed with appsrc.
  *
  * The main way of handing data to the appsrc element is by calling the
  * gst_app_src_push_buffer() method or by emiting the push-buffer action signal.
@@ -99,10 +96,10 @@
  * gst_app_src_end_of_stream() or emit the end-of-stream action signal. After
  * this call, no more buffers can be pushed into appsrc until a flushing seek
  * happened or the state of the appsrc has gone through READY.
+ * 
+ * Since: 0.10.22
  *
  * Last reviewed on 2008-12-17 (0.10.10)
- *
- * Since: 0.10.22
  */
 
 #ifdef HAVE_CONFIG_H
@@ -139,12 +136,6 @@ struct _GstAppSrcPrivate
 
   guint64 min_latency;
   guint64 max_latency;
-  gboolean emit_signals;
-  guint min_percent;
-
-  GstAppSrcCallbacks callbacks;
-  gpointer user_data;
-  GDestroyNotify notify;
 };
 
 GST_DEBUG_CATEGORY_STATIC (app_src_debug);
@@ -172,8 +163,6 @@ enum
 #define DEFAULT_PROP_IS_LIVE       FALSE
 #define DEFAULT_PROP_MIN_LATENCY   -1
 #define DEFAULT_PROP_MAX_LATENCY   -1
-#define DEFAULT_PROP_EMIT_SIGNALS  TRUE
-#define DEFAULT_PROP_MIN_PERCENT   0
 
 enum
 {
@@ -187,8 +176,6 @@ enum
   PROP_IS_LIVE,
   PROP_MIN_LATENCY,
   PROP_MAX_LATENCY,
-  PROP_EMIT_SIGNALS,
-  PROP_MIN_PERCENT,
   PROP_LAST
 };
 
@@ -389,34 +376,6 @@ gst_app_src_class_init (GstAppSrcClass * klass)
           -1, G_MAXINT64, DEFAULT_PROP_MAX_LATENCY,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  /**
-   * GstAppSrc::emit-signals
-   *
-   * Make appsrc emit the "need-data", "enough-data" and "seek-data" signals.
-   * This option is by default enabled for backwards compatibility reasons but
-   * can disabled when needed because signal emission is expensive.
-   *
-   * Since: 0.10.23
-   */
-  g_object_class_install_property (gobject_class, PROP_EMIT_SIGNALS,
-      g_param_spec_boolean ("emit-signals", "Emit signals",
-          "Emit new-preroll and new-buffer signals", DEFAULT_PROP_EMIT_SIGNALS,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  /**
-   * GstAppSrc::empty-percent
-   *
-   * Make appsrc emit the "need-data" signal when the amount of bytes in the
-   * queue drops below this percentage of max-bytes.
-   *
-   * Since: 0.10.27
-   */
-  g_object_class_install_property (gobject_class, PROP_MIN_PERCENT,
-      g_param_spec_uint ("min-percent", "Min Percent",
-          "Emit need-data when queued bytes drops below this percent of max-bytes",
-          0, 100, DEFAULT_PROP_MIN_PERCENT,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
   /**
    * GstAppSrc::need-data:
    * @appsrc: the appsrc element that emited the signal
@@ -434,7 +393,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
   gst_app_src_signals[SIGNAL_NEED_DATA] =
       g_signal_new ("need-data", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstAppSrcClass, need_data),
-      NULL, NULL, __gst_app_marshal_VOID__UINT, G_TYPE_NONE, 1, G_TYPE_UINT);
+      NULL, NULL, gst_app_marshal_VOID__UINT, G_TYPE_NONE, 1, G_TYPE_UINT);
 
   /**
    * GstAppSrc::enough-data:
@@ -463,7 +422,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
   gst_app_src_signals[SIGNAL_SEEK_DATA] =
       g_signal_new ("seek-data", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstAppSrcClass, seek_data),
-      NULL, NULL, __gst_app_marshal_BOOLEAN__UINT64, G_TYPE_BOOLEAN, 1,
+      NULL, NULL, gst_app_marshal_BOOLEAN__UINT64, G_TYPE_BOOLEAN, 1,
       G_TYPE_UINT64);
 
    /**
@@ -481,7 +440,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
   gst_app_src_signals[SIGNAL_PUSH_BUFFER] =
       g_signal_new ("push-buffer", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (GstAppSrcClass,
-          push_buffer), NULL, NULL, __gst_app_marshal_ENUM__OBJECT,
+          push_buffer), NULL, NULL, gst_app_marshal_ENUM__OBJECT,
       GST_TYPE_FLOW_RETURN, 1, GST_TYPE_BUFFER);
 
    /**
@@ -493,7 +452,7 @@ gst_app_src_class_init (GstAppSrcClass * klass)
   gst_app_src_signals[SIGNAL_END_OF_STREAM] =
       g_signal_new ("end-of-stream", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, G_STRUCT_OFFSET (GstAppSrcClass,
-          end_of_stream), NULL, NULL, __gst_app_marshal_ENUM__VOID,
+          end_of_stream), NULL, NULL, gst_app_marshal_ENUM__VOID,
       GST_TYPE_FLOW_RETURN, 0, G_TYPE_NONE);
 
   basesrc_class->create = gst_app_src_create;
@@ -517,24 +476,20 @@ gst_app_src_class_init (GstAppSrcClass * klass)
 static void
 gst_app_src_init (GstAppSrc * appsrc, GstAppSrcClass * klass)
 {
-  GstAppSrcPrivate *priv;
-
-  priv = appsrc->priv = G_TYPE_INSTANCE_GET_PRIVATE (appsrc, GST_TYPE_APP_SRC,
+  appsrc->priv = G_TYPE_INSTANCE_GET_PRIVATE (appsrc, GST_TYPE_APP_SRC,
       GstAppSrcPrivate);
 
-  priv->mutex = g_mutex_new ();
-  priv->cond = g_cond_new ();
-  priv->queue = g_queue_new ();
+  appsrc->priv->mutex = g_mutex_new ();
+  appsrc->priv->cond = g_cond_new ();
+  appsrc->priv->queue = g_queue_new ();
 
-  priv->size = DEFAULT_PROP_SIZE;
-  priv->stream_type = DEFAULT_PROP_STREAM_TYPE;
-  priv->max_bytes = DEFAULT_PROP_MAX_BYTES;
-  priv->format = DEFAULT_PROP_FORMAT;
-  priv->block = DEFAULT_PROP_BLOCK;
-  priv->min_latency = DEFAULT_PROP_MIN_LATENCY;
-  priv->max_latency = DEFAULT_PROP_MAX_LATENCY;
-  priv->emit_signals = DEFAULT_PROP_EMIT_SIGNALS;
-  priv->min_percent = DEFAULT_PROP_MIN_PERCENT;
+  appsrc->priv->size = DEFAULT_PROP_SIZE;
+  appsrc->priv->stream_type = DEFAULT_PROP_STREAM_TYPE;
+  appsrc->priv->max_bytes = DEFAULT_PROP_MAX_BYTES;
+  appsrc->priv->format = DEFAULT_PROP_FORMAT;
+  appsrc->priv->block = DEFAULT_PROP_BLOCK;
+  appsrc->priv->min_latency = DEFAULT_PROP_MIN_LATENCY;
+  appsrc->priv->max_latency = DEFAULT_PROP_MAX_LATENCY;
 
   gst_base_src_set_live (GST_BASE_SRC (appsrc), DEFAULT_PROP_IS_LIVE);
 }
@@ -543,22 +498,19 @@ static void
 gst_app_src_flush_queued (GstAppSrc * src)
 {
   GstBuffer *buf;
-  GstAppSrcPrivate *priv = src->priv;
 
-  while ((buf = g_queue_pop_head (priv->queue)))
+  while ((buf = g_queue_pop_head (src->priv->queue)))
     gst_buffer_unref (buf);
-  priv->queued_bytes = 0;
 }
 
 static void
 gst_app_src_dispose (GObject * obj)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (obj);
-  GstAppSrcPrivate *priv = appsrc->priv;
+  GstAppSrc *appsrc = GST_APP_SRC (obj);
 
-  if (priv->caps) {
-    gst_caps_unref (priv->caps);
-    priv->caps = NULL;
+  if (appsrc->priv->caps) {
+    gst_caps_unref (appsrc->priv->caps);
+    appsrc->priv->caps = NULL;
   }
   gst_app_src_flush_queued (appsrc);
 
@@ -568,12 +520,11 @@ gst_app_src_dispose (GObject * obj)
 static void
 gst_app_src_finalize (GObject * obj)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (obj);
-  GstAppSrcPrivate *priv = appsrc->priv;
+  GstAppSrc *appsrc = GST_APP_SRC (obj);
 
-  g_mutex_free (priv->mutex);
-  g_cond_free (priv->cond);
-  g_queue_free (priv->queue);
+  g_mutex_free (appsrc->priv->mutex);
+  g_cond_free (appsrc->priv->cond);
+  g_queue_free (appsrc->priv->queue);
 
   G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
@@ -582,8 +533,7 @@ static void
 gst_app_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (object);
-  GstAppSrcPrivate *priv = appsrc->priv;
+  GstAppSrc *appsrc = GST_APP_SRC (object);
 
   switch (prop_id) {
     case PROP_CAPS:
@@ -599,10 +549,10 @@ gst_app_src_set_property (GObject * object, guint prop_id,
       gst_app_src_set_max_bytes (appsrc, g_value_get_uint64 (value));
       break;
     case PROP_FORMAT:
-      priv->format = g_value_get_enum (value);
+      appsrc->priv->format = g_value_get_enum (value);
       break;
     case PROP_BLOCK:
-      priv->block = g_value_get_boolean (value);
+      appsrc->priv->block = g_value_get_boolean (value);
       break;
     case PROP_IS_LIVE:
       gst_base_src_set_live (GST_BASE_SRC (appsrc),
@@ -616,12 +566,6 @@ gst_app_src_set_property (GObject * object, guint prop_id,
       gst_app_src_set_latencies (appsrc, FALSE, -1, TRUE,
           g_value_get_int64 (value));
       break;
-    case PROP_EMIT_SIGNALS:
-      gst_app_src_set_emit_signals (appsrc, g_value_get_boolean (value));
-      break;
-    case PROP_MIN_PERCENT:
-      priv->min_percent = g_value_get_uint (value);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -632,8 +576,7 @@ static void
 gst_app_src_get_property (GObject * object, guint prop_id, GValue * value,
     GParamSpec * pspec)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (object);
-  GstAppSrcPrivate *priv = appsrc->priv;
+  GstAppSrc *appsrc = GST_APP_SRC (object);
 
   switch (prop_id) {
     case PROP_CAPS:
@@ -657,10 +600,10 @@ gst_app_src_get_property (GObject * object, guint prop_id, GValue * value,
       g_value_set_uint64 (value, gst_app_src_get_max_bytes (appsrc));
       break;
     case PROP_FORMAT:
-      g_value_set_enum (value, priv->format);
+      g_value_set_enum (value, appsrc->priv->format);
       break;
     case PROP_BLOCK:
-      g_value_set_boolean (value, priv->block);
+      g_value_set_boolean (value, appsrc->priv->block);
       break;
     case PROP_IS_LIVE:
       g_value_set_boolean (value, gst_base_src_is_live (GST_BASE_SRC (appsrc)));
@@ -681,12 +624,6 @@ gst_app_src_get_property (GObject * object, guint prop_id, GValue * value,
       g_value_set_int64 (value, max);
       break;
     }
-    case PROP_EMIT_SIGNALS:
-      g_value_set_boolean (value, gst_app_src_get_emit_signals (appsrc));
-      break;
-    case PROP_MIN_PERCENT:
-      g_value_set_uint (value, priv->min_percent);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -696,14 +633,13 @@ gst_app_src_get_property (GObject * object, guint prop_id, GValue * value,
 static gboolean
 gst_app_src_unlock (GstBaseSrc * bsrc)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (bsrc);
-  GstAppSrcPrivate *priv = appsrc->priv;
+  GstAppSrc *appsrc = GST_APP_SRC (bsrc);
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (appsrc->priv->mutex);
   GST_DEBUG_OBJECT (appsrc, "unlock start");
-  priv->flushing = TRUE;
-  g_cond_broadcast (priv->cond);
-  g_mutex_unlock (priv->mutex);
+  appsrc->priv->flushing = TRUE;
+  g_cond_broadcast (appsrc->priv->cond);
+  g_mutex_unlock (appsrc->priv->mutex);
 
   return TRUE;
 }
@@ -711,14 +647,13 @@ gst_app_src_unlock (GstBaseSrc * bsrc)
 static gboolean
 gst_app_src_unlock_stop (GstBaseSrc * bsrc)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (bsrc);
-  GstAppSrcPrivate *priv = appsrc->priv;
+  GstAppSrc *appsrc = GST_APP_SRC (bsrc);
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (appsrc->priv->mutex);
   GST_DEBUG_OBJECT (appsrc, "unlock stop");
-  priv->flushing = FALSE;
-  g_cond_broadcast (priv->cond);
-  g_mutex_unlock (priv->mutex);
+  appsrc->priv->flushing = FALSE;
+  g_cond_broadcast (appsrc->priv->cond);
+  g_mutex_unlock (appsrc->priv->mutex);
 
   return TRUE;
 }
@@ -726,19 +661,18 @@ gst_app_src_unlock_stop (GstBaseSrc * bsrc)
 static gboolean
 gst_app_src_start (GstBaseSrc * bsrc)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (bsrc);
-  GstAppSrcPrivate *priv = appsrc->priv;
+  GstAppSrc *appsrc = GST_APP_SRC (bsrc);
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (appsrc->priv->mutex);
   GST_DEBUG_OBJECT (appsrc, "starting");
-  priv->started = TRUE;
+  appsrc->priv->started = TRUE;
   /* set the offset to -1 so that we always do a first seek. This is only used
    * in random-access mode. */
-  priv->offset = -1;
-  priv->flushing = FALSE;
-  g_mutex_unlock (priv->mutex);
+  appsrc->priv->offset = -1;
+  appsrc->priv->flushing = FALSE;
+  g_mutex_unlock (appsrc->priv->mutex);
 
-  gst_base_src_set_format (bsrc, priv->format);
+  gst_base_src_set_format (bsrc, appsrc->priv->format);
 
   return TRUE;
 }
@@ -746,16 +680,15 @@ gst_app_src_start (GstBaseSrc * bsrc)
 static gboolean
 gst_app_src_stop (GstBaseSrc * bsrc)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (bsrc);
-  GstAppSrcPrivate *priv = appsrc->priv;
+  GstAppSrc *appsrc = GST_APP_SRC (bsrc);
 
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (appsrc->priv->mutex);
   GST_DEBUG_OBJECT (appsrc, "stopping");
-  priv->is_eos = FALSE;
-  priv->flushing = TRUE;
-  priv->started = FALSE;
+  appsrc->priv->is_eos = FALSE;
+  appsrc->priv->flushing = TRUE;
+  appsrc->priv->started = FALSE;
   gst_app_src_flush_queued (appsrc);
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (appsrc->priv->mutex);
 
   return TRUE;
 }
@@ -763,11 +696,10 @@ gst_app_src_stop (GstBaseSrc * bsrc)
 static gboolean
 gst_app_src_is_seekable (GstBaseSrc * src)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (src);
-  GstAppSrcPrivate *priv = appsrc->priv;
+  GstAppSrc *appsrc = GST_APP_SRC (src);
   gboolean res = FALSE;
 
-  switch (priv->stream_type) {
+  switch (appsrc->priv->stream_type) {
     case GST_APP_STREAM_TYPE_STREAM:
       break;
     case GST_APP_STREAM_TYPE_SEEKABLE:
@@ -781,11 +713,10 @@ gst_app_src_is_seekable (GstBaseSrc * src)
 static gboolean
 gst_app_src_check_get_range (GstBaseSrc * src)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (src);
-  GstAppSrcPrivate *priv = appsrc->priv;
+  GstAppSrc *appsrc = GST_APP_SRC (src);
   gboolean res = FALSE;
 
-  switch (priv->stream_type) {
+  switch (appsrc->priv->stream_type) {
     case GST_APP_STREAM_TYPE_STREAM:
     case GST_APP_STREAM_TYPE_SEEKABLE:
       break;
@@ -799,7 +730,7 @@ gst_app_src_check_get_range (GstBaseSrc * src)
 static gboolean
 gst_app_src_do_get_size (GstBaseSrc * src, guint64 * size)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (src);
+  GstAppSrc *appsrc = GST_APP_SRC (src);
 
   *size = gst_app_src_get_size (appsrc);
 
@@ -809,8 +740,7 @@ gst_app_src_do_get_size (GstBaseSrc * src, guint64 * size)
 static gboolean
 gst_app_src_query (GstBaseSrc * src, GstQuery * query)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (src);
-  GstAppSrcPrivate *priv = appsrc->priv;
+  GstAppSrc *appsrc = GST_APP_SRC (src);
   gboolean res;
 
   switch (GST_QUERY_TYPE (query)) {
@@ -823,12 +753,12 @@ gst_app_src_query (GstBaseSrc * src, GstQuery * query)
       res = gst_base_src_query_latency (src, &live, &min, &max);
 
       /* overwrite with our values when we need to */
-      g_mutex_lock (priv->mutex);
-      if (priv->min_latency != -1)
-        min = priv->min_latency;
-      if (priv->max_latency != -1)
-        max = priv->max_latency;
-      g_mutex_unlock (priv->mutex);
+      g_mutex_lock (appsrc->priv->mutex);
+      if (appsrc->priv->min_latency != -1)
+        min = appsrc->priv->min_latency;
+      if (appsrc->priv->max_latency != -1)
+        max = appsrc->priv->max_latency;
+      g_mutex_unlock (appsrc->priv->mutex);
 
       gst_query_set_latency (query, live, min, max);
       break;
@@ -845,8 +775,7 @@ gst_app_src_query (GstBaseSrc * src, GstQuery * query)
 static gboolean
 gst_app_src_do_seek (GstBaseSrc * src, GstSegment * segment)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (src);
-  GstAppSrcPrivate *priv = appsrc->priv;
+  GstAppSrc *appsrc = GST_APP_SRC (src);
   gint64 desired_position;
   gboolean res = FALSE;
 
@@ -856,27 +785,15 @@ gst_app_src_do_seek (GstBaseSrc * src, GstSegment * segment)
       desired_position, gst_format_get_name (segment->format));
 
   /* no need to try to seek in streaming mode */
-  if (priv->stream_type == GST_APP_STREAM_TYPE_STREAM)
+  if (appsrc->priv->stream_type == GST_APP_STREAM_TYPE_STREAM)
     return TRUE;
 
-  if (priv->callbacks.seek_data)
-    res = priv->callbacks.seek_data (appsrc, desired_position, priv->user_data);
-  else {
-    gboolean emit;
-
-    g_mutex_lock (priv->mutex);
-    emit = priv->emit_signals;
-    g_mutex_unlock (priv->mutex);
-
-    if (emit)
-      g_signal_emit (appsrc, gst_app_src_signals[SIGNAL_SEEK_DATA], 0,
-          desired_position, &res);
-  }
+  g_signal_emit (appsrc, gst_app_src_signals[SIGNAL_SEEK_DATA], 0,
+      desired_position, &res);
 
   if (res) {
     GST_DEBUG_OBJECT (appsrc, "flushing queue");
     gst_app_src_flush_queued (appsrc);
-    priv->is_eos = FALSE;
   } else {
     GST_WARNING_OBJECT (appsrc, "seek failed");
   }
@@ -884,134 +801,77 @@ gst_app_src_do_seek (GstBaseSrc * src, GstSegment * segment)
   return res;
 }
 
-/* must be called with the appsrc mutex */
-static gboolean
-gst_app_src_emit_seek (GstAppSrc * appsrc, guint64 offset)
-{
-  gboolean res;
-  gboolean emit;
-  GstAppSrcPrivate *priv = appsrc->priv;
-
-  emit = priv->emit_signals;
-  g_mutex_unlock (priv->mutex);
-
-  GST_DEBUG_OBJECT (appsrc,
-      "we are at %" G_GINT64_FORMAT ", seek to %" G_GINT64_FORMAT,
-      priv->offset, offset);
-
-  if (priv->callbacks.seek_data)
-    res = priv->callbacks.seek_data (appsrc, offset, priv->user_data);
-  else if (emit)
-    g_signal_emit (appsrc, gst_app_src_signals[SIGNAL_SEEK_DATA], 0,
-        offset, &res);
-
-  g_mutex_lock (priv->mutex);
-
-  return res;
-}
-
-/* must be called with the appsrc mutex. After this call things can be
- * flushing */
-static void
-gst_app_src_emit_need_data (GstAppSrc * appsrc, guint size)
-{
-  gboolean emit;
-  GstAppSrcPrivate *priv = appsrc->priv;
-
-  emit = priv->emit_signals;
-  g_mutex_unlock (priv->mutex);
-
-  /* we have no data, we need some. We fire the signal with the size hint. */
-  if (priv->callbacks.need_data)
-    priv->callbacks.need_data (appsrc, size, priv->user_data);
-  else if (emit)
-    g_signal_emit (appsrc, gst_app_src_signals[SIGNAL_NEED_DATA], 0, size,
-        NULL);
-
-  g_mutex_lock (priv->mutex);
-  /* we can be flushing now because we released the lock */
-}
-
 static GstFlowReturn
 gst_app_src_create (GstBaseSrc * bsrc, guint64 offset, guint size,
     GstBuffer ** buf)
 {
-  GstAppSrc *appsrc = GST_APP_SRC_CAST (bsrc);
-  GstAppSrcPrivate *priv = appsrc->priv;
+  GstAppSrc *appsrc = GST_APP_SRC (bsrc);
   GstFlowReturn ret;
 
-  GST_OBJECT_LOCK (appsrc);
-  if (G_UNLIKELY (priv->size != bsrc->segment.duration &&
-          bsrc->segment.format == GST_FORMAT_BYTES)) {
-    GST_DEBUG_OBJECT (appsrc,
-        "Size changed from %" G_GINT64_FORMAT " to %" G_GINT64_FORMAT,
-        bsrc->segment.duration, priv->size);
-    gst_segment_set_duration (&bsrc->segment, GST_FORMAT_BYTES, priv->size);
-    GST_OBJECT_UNLOCK (appsrc);
-
-    gst_element_post_message (GST_ELEMENT (appsrc),
-        gst_message_new_duration (GST_OBJECT (appsrc), GST_FORMAT_BYTES,
-            priv->size));
-  } else {
-    GST_OBJECT_UNLOCK (appsrc);
-  }
-
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (appsrc->priv->mutex);
   /* check flushing first */
-  if (G_UNLIKELY (priv->flushing))
+  if (G_UNLIKELY (appsrc->priv->flushing))
     goto flushing;
 
-  if (priv->stream_type == GST_APP_STREAM_TYPE_RANDOM_ACCESS) {
+  if (appsrc->priv->stream_type == GST_APP_STREAM_TYPE_RANDOM_ACCESS) {
     /* if we are dealing with a random-access stream, issue a seek if the offset
      * changed. */
-    if (G_UNLIKELY (priv->offset != offset)) {
+    if (G_UNLIKELY (appsrc->priv->offset != offset)) {
       gboolean res;
 
-      /* do the seek */
-      res = gst_app_src_emit_seek (appsrc, offset);
+      g_mutex_unlock (appsrc->priv->mutex);
+
+      GST_DEBUG_OBJECT (appsrc,
+          "we are at %" G_GINT64_FORMAT ", seek to %" G_GINT64_FORMAT,
+          appsrc->priv->offset, offset);
+
+      g_signal_emit (appsrc, gst_app_src_signals[SIGNAL_SEEK_DATA], 0,
+          offset, &res);
 
       if (G_UNLIKELY (!res))
         /* failing to seek is fatal */
         goto seek_error;
 
-      priv->offset = offset;
+      g_mutex_lock (appsrc->priv->mutex);
+
+      appsrc->priv->offset = offset;
     }
   }
 
   while (TRUE) {
     /* return data as long as we have some */
-    if (!g_queue_is_empty (priv->queue)) {
+    if (!g_queue_is_empty (appsrc->priv->queue)) {
       guint buf_size;
 
-      *buf = g_queue_pop_head (priv->queue);
+      *buf = g_queue_pop_head (appsrc->priv->queue);
       buf_size = GST_BUFFER_SIZE (*buf);
 
       GST_DEBUG_OBJECT (appsrc, "we have buffer %p of size %u", *buf, buf_size);
 
-      priv->queued_bytes -= buf_size;
+      appsrc->priv->queued_bytes -= buf_size;
 
       /* only update the offset when in random_access mode */
-      if (priv->stream_type == GST_APP_STREAM_TYPE_RANDOM_ACCESS)
-        priv->offset += buf_size;
-      gst_buffer_set_caps (*buf, priv->caps);
+      if (appsrc->priv->stream_type == GST_APP_STREAM_TYPE_RANDOM_ACCESS) {
+        appsrc->priv->offset += buf_size;
+      }
+
+      gst_buffer_set_caps (*buf, appsrc->priv->caps);
 
       /* signal that we removed an item */
-      g_cond_broadcast (priv->cond);
+      g_cond_broadcast (appsrc->priv->cond);
 
-      /* see if we go lower than the empty-percent */
-      if (priv->min_percent && priv->max_bytes) {
-        if (priv->queued_bytes * 100 / priv->max_bytes <= priv->min_percent)
-          /* ignore flushing state, we got a buffer and we will return it now.
-           * Errors will be handled in the next round */
-          gst_app_src_emit_need_data (appsrc, size);
-      }
       ret = GST_FLOW_OK;
       break;
     } else {
-      gst_app_src_emit_need_data (appsrc, size);
+      g_mutex_unlock (appsrc->priv->mutex);
 
-      /* we can be flushing now because we released the lock above */
-      if (G_UNLIKELY (priv->flushing))
+      /* we have no data, we need some. We fire the signal with the size hint. */
+      g_signal_emit (appsrc, gst_app_src_signals[SIGNAL_NEED_DATA], 0, size,
+          NULL);
+
+      g_mutex_lock (appsrc->priv->mutex);
+      /* we can be flushing now because we released the lock */
+      if (G_UNLIKELY (appsrc->priv->flushing))
         goto flushing;
 
       /* if we have a buffer now, continue the loop and try to return it. In
@@ -1019,20 +879,20 @@ gst_app_src_create (GstBaseSrc * bsrc, guint64 offset, guint size,
        * signal) we can still be empty because the pushed buffer got flushed or
        * when the application pushes the requested buffer later, we support both
        * possiblities. */
-      if (!g_queue_is_empty (priv->queue))
+      if (!g_queue_is_empty (appsrc->priv->queue))
         continue;
 
       /* no buffer yet, maybe we are EOS, if not, block for more data. */
     }
 
     /* check EOS */
-    if (G_UNLIKELY (priv->is_eos))
+    if (G_UNLIKELY (appsrc->priv->is_eos))
       goto eos;
 
     /* nothing to return, wait a while for new data or flushing. */
-    g_cond_wait (priv->cond, priv->mutex);
+    g_cond_wait (appsrc->priv->cond, appsrc->priv->mutex);
   }
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (appsrc->priv->mutex);
 
   return ret;
 
@@ -1040,18 +900,17 @@ gst_app_src_create (GstBaseSrc * bsrc, guint64 offset, guint size,
 flushing:
   {
     GST_DEBUG_OBJECT (appsrc, "we are flushing");
-    g_mutex_unlock (priv->mutex);
+    g_mutex_unlock (appsrc->priv->mutex);
     return GST_FLOW_WRONG_STATE;
   }
 eos:
   {
     GST_DEBUG_OBJECT (appsrc, "we are EOS");
-    g_mutex_unlock (priv->mutex);
+    g_mutex_unlock (appsrc->priv->mutex);
     return GST_FLOW_UNEXPECTED;
   }
 seek_error:
   {
-    g_mutex_unlock (priv->mutex);
     GST_ELEMENT_ERROR (appsrc, RESOURCE, READ, ("failed to seek"),
         GST_ERROR_SYSTEM);
     return GST_FLOW_ERROR;
@@ -1076,19 +935,16 @@ void
 gst_app_src_set_caps (GstAppSrc * appsrc, const GstCaps * caps)
 {
   GstCaps *old;
-  GstAppSrcPrivate *priv;
 
   g_return_if_fail (GST_IS_APP_SRC (appsrc));
 
-  priv = appsrc->priv;
-
   GST_OBJECT_LOCK (appsrc);
   GST_DEBUG_OBJECT (appsrc, "setting caps to %" GST_PTR_FORMAT, caps);
-  if ((old = priv->caps) != caps) {
+  if ((old = appsrc->priv->caps) != caps) {
     if (caps)
-      priv->caps = gst_caps_copy (caps);
+      appsrc->priv->caps = gst_caps_copy (caps);
     else
-      priv->caps = NULL;
+      appsrc->priv->caps = NULL;
     if (old)
       gst_caps_unref (old);
   }
@@ -1109,14 +965,12 @@ GstCaps *
 gst_app_src_get_caps (GstAppSrc * appsrc)
 {
   GstCaps *caps;
-  GstAppSrcPrivate *priv;
 
+  g_return_val_if_fail (appsrc != NULL, NULL);
   g_return_val_if_fail (GST_IS_APP_SRC (appsrc), NULL);
 
-  priv = appsrc->priv;
-
   GST_OBJECT_LOCK (appsrc);
-  if ((caps = priv->caps))
+  if ((caps = appsrc->priv->caps))
     gst_caps_ref (caps);
   GST_DEBUG_OBJECT (appsrc, "getting caps of %" GST_PTR_FORMAT, caps);
   GST_OBJECT_UNLOCK (appsrc);
@@ -1137,15 +991,12 @@ gst_app_src_get_caps (GstAppSrc * appsrc)
 void
 gst_app_src_set_size (GstAppSrc * appsrc, gint64 size)
 {
-  GstAppSrcPrivate *priv;
-
+  g_return_if_fail (appsrc != NULL);
   g_return_if_fail (GST_IS_APP_SRC (appsrc));
 
-  priv = appsrc->priv;
-
   GST_OBJECT_LOCK (appsrc);
   GST_DEBUG_OBJECT (appsrc, "setting size of %" G_GINT64_FORMAT, size);
-  priv->size = size;
+  appsrc->priv->size = size;
   GST_OBJECT_UNLOCK (appsrc);
 }
 
@@ -1164,14 +1015,12 @@ gint64
 gst_app_src_get_size (GstAppSrc * appsrc)
 {
   gint64 size;
-  GstAppSrcPrivate *priv;
 
+  g_return_val_if_fail (appsrc != NULL, -1);
   g_return_val_if_fail (GST_IS_APP_SRC (appsrc), -1);
 
-  priv = appsrc->priv;
-
   GST_OBJECT_LOCK (appsrc);
-  size = priv->size;
+  size = appsrc->priv->size;
   GST_DEBUG_OBJECT (appsrc, "getting size of %" G_GINT64_FORMAT, size);
   GST_OBJECT_UNLOCK (appsrc);
 
@@ -1193,15 +1042,12 @@ gst_app_src_get_size (GstAppSrc * appsrc)
 void
 gst_app_src_set_stream_type (GstAppSrc * appsrc, GstAppStreamType type)
 {
-  GstAppSrcPrivate *priv;
-
+  g_return_if_fail (appsrc != NULL);
   g_return_if_fail (GST_IS_APP_SRC (appsrc));
 
-  priv = appsrc->priv;
-
   GST_OBJECT_LOCK (appsrc);
   GST_DEBUG_OBJECT (appsrc, "setting stream_type of %d", type);
-  priv->stream_type = type;
+  appsrc->priv->stream_type = type;
   GST_OBJECT_UNLOCK (appsrc);
 }
 
@@ -1220,14 +1066,12 @@ GstAppStreamType
 gst_app_src_get_stream_type (GstAppSrc * appsrc)
 {
   gboolean stream_type;
-  GstAppSrcPrivate *priv;
 
+  g_return_val_if_fail (appsrc != NULL, FALSE);
   g_return_val_if_fail (GST_IS_APP_SRC (appsrc), FALSE);
 
-  priv = appsrc->priv;
-
   GST_OBJECT_LOCK (appsrc);
-  stream_type = priv->stream_type;
+  stream_type = appsrc->priv->stream_type;
   GST_DEBUG_OBJECT (appsrc, "getting stream_type of %d", stream_type);
   GST_OBJECT_UNLOCK (appsrc);
 
@@ -1248,20 +1092,16 @@ gst_app_src_get_stream_type (GstAppSrc * appsrc)
 void
 gst_app_src_set_max_bytes (GstAppSrc * appsrc, guint64 max)
 {
-  GstAppSrcPrivate *priv;
-
   g_return_if_fail (GST_IS_APP_SRC (appsrc));
 
-  priv = appsrc->priv;
-
-  g_mutex_lock (priv->mutex);
-  if (max != priv->max_bytes) {
+  g_mutex_lock (appsrc->priv->mutex);
+  if (max != appsrc->priv->max_bytes) {
     GST_DEBUG_OBJECT (appsrc, "setting max-bytes to %" G_GUINT64_FORMAT, max);
-    priv->max_bytes = max;
+    appsrc->priv->max_bytes = max;
     /* signal the change */
-    g_cond_broadcast (priv->cond);
+    g_cond_broadcast (appsrc->priv->cond);
   }
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (appsrc->priv->mutex);
 }
 
 /**
@@ -1278,16 +1118,13 @@ guint64
 gst_app_src_get_max_bytes (GstAppSrc * appsrc)
 {
   guint64 result;
-  GstAppSrcPrivate *priv;
 
   g_return_val_if_fail (GST_IS_APP_SRC (appsrc), 0);
 
-  priv = appsrc->priv;
-
-  g_mutex_lock (priv->mutex);
-  result = priv->max_bytes;
+  g_mutex_lock (appsrc->priv->mutex);
+  result = appsrc->priv->max_bytes;
   GST_DEBUG_OBJECT (appsrc, "getting max-bytes of %" G_GUINT64_FORMAT, result);
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (appsrc->priv->mutex);
 
   return result;
 }
@@ -1296,19 +1133,18 @@ static void
 gst_app_src_set_latencies (GstAppSrc * appsrc, gboolean do_min, guint64 min,
     gboolean do_max, guint64 max)
 {
-  GstAppSrcPrivate *priv = appsrc->priv;
   gboolean changed = FALSE;
 
-  g_mutex_lock (priv->mutex);
-  if (do_min && priv->min_latency != min) {
-    priv->min_latency = min;
+  g_mutex_lock (appsrc->priv->mutex);
+  if (do_min && appsrc->priv->min_latency != min) {
+    appsrc->priv->min_latency = min;
     changed = TRUE;
   }
-  if (do_max && priv->max_latency != max) {
-    priv->max_latency = max;
+  if (do_max && appsrc->priv->max_latency != max) {
+    appsrc->priv->max_latency = max;
     changed = TRUE;
   }
-  g_mutex_unlock (priv->mutex);
+  g_mutex_unlock (appsrc->priv->mutex);
 
   if (changed) {
     GST_DEBUG_OBJECT (appsrc, "posting latency changed");
@@ -1347,71 +1183,14 @@ gst_app_src_set_latency (GstAppSrc * appsrc, guint64 min, guint64 max)
 void
 gst_app_src_get_latency (GstAppSrc * appsrc, guint64 * min, guint64 * max)
 {
-  GstAppSrcPrivate *priv;
-
   g_return_if_fail (GST_IS_APP_SRC (appsrc));
 
-  priv = appsrc->priv;
-
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (appsrc->priv->mutex);
   if (min)
-    *min = priv->min_latency;
+    *min = appsrc->priv->min_latency;
   if (max)
-    *max = priv->max_latency;
-  g_mutex_unlock (priv->mutex);
-}
-
-/**
- * gst_app_src_set_emit_signals:
- * @appsrc: a #GstAppSrc
- * @emit: the new state
- *
- * Make appsrc emit the "new-preroll" and "new-buffer" signals. This option is
- * by default disabled because signal emission is expensive and unneeded when
- * the application prefers to operate in pull mode.
- *
- * Since: 0.10.23
- */
-void
-gst_app_src_set_emit_signals (GstAppSrc * appsrc, gboolean emit)
-{
-  GstAppSrcPrivate *priv;
-
-  g_return_if_fail (GST_IS_APP_SRC (appsrc));
-
-  priv = appsrc->priv;
-
-  g_mutex_lock (priv->mutex);
-  priv->emit_signals = emit;
-  g_mutex_unlock (priv->mutex);
-}
-
-/**
- * gst_app_src_get_emit_signals:
- * @appsrc: a #GstAppSrc
- *
- * Check if appsrc will emit the "new-preroll" and "new-buffer" signals.
- *
- * Returns: %TRUE if @appsrc is emiting the "new-preroll" and "new-buffer"
- * signals.
- *
- * Since: 0.10.23
- */
-gboolean
-gst_app_src_get_emit_signals (GstAppSrc * appsrc)
-{
-  gboolean result;
-  GstAppSrcPrivate *priv;
-
-  g_return_val_if_fail (GST_IS_APP_SRC (appsrc), FALSE);
-
-  priv = appsrc->priv;
-
-  g_mutex_lock (priv->mutex);
-  result = priv->emit_signals;
-  g_mutex_unlock (priv->mutex);
-
-  return result;
+    *max = appsrc->priv->max_latency;
+  g_mutex_unlock (appsrc->priv->mutex);
 }
 
 static GstFlowReturn
@@ -1419,51 +1198,44 @@ gst_app_src_push_buffer_full (GstAppSrc * appsrc, GstBuffer * buffer,
     gboolean steal_ref)
 {
   gboolean first = TRUE;
-  GstAppSrcPrivate *priv;
 
+  g_return_val_if_fail (appsrc, GST_FLOW_ERROR);
   g_return_val_if_fail (GST_IS_APP_SRC (appsrc), GST_FLOW_ERROR);
   g_return_val_if_fail (GST_IS_BUFFER (buffer), GST_FLOW_ERROR);
 
-  priv = appsrc->priv;
-
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (appsrc->priv->mutex);
 
   while (TRUE) {
     /* can't accept buffers when we are flushing or EOS */
-    if (priv->flushing)
+    if (appsrc->priv->flushing)
       goto flushing;
 
-    if (priv->is_eos)
+    if (appsrc->priv->is_eos)
       goto eos;
 
-    if (priv->max_bytes && priv->queued_bytes >= priv->max_bytes) {
+    if (appsrc->priv->max_bytes
+        && appsrc->priv->queued_bytes >= appsrc->priv->max_bytes) {
       GST_DEBUG_OBJECT (appsrc,
           "queue filled (%" G_GUINT64_FORMAT " >= %" G_GUINT64_FORMAT ")",
-          priv->queued_bytes, priv->max_bytes);
+          appsrc->priv->queued_bytes, appsrc->priv->max_bytes);
 
       if (first) {
-        gboolean emit;
-
-        emit = priv->emit_signals;
         /* only signal on the first push */
-        g_mutex_unlock (priv->mutex);
+        g_mutex_unlock (appsrc->priv->mutex);
 
-        if (priv->callbacks.enough_data)
-          priv->callbacks.enough_data (appsrc, priv->user_data);
-        else if (emit)
-          g_signal_emit (appsrc, gst_app_src_signals[SIGNAL_ENOUGH_DATA], 0,
-              NULL);
+        g_signal_emit (appsrc, gst_app_src_signals[SIGNAL_ENOUGH_DATA], 0,
+            NULL);
 
-        g_mutex_lock (priv->mutex);
+        g_mutex_lock (appsrc->priv->mutex);
         /* continue to check for flushing/eos after releasing the lock */
         first = FALSE;
         continue;
       }
-      if (priv->block) {
+      if (appsrc->priv->block) {
         GST_DEBUG_OBJECT (appsrc, "waiting for free space");
         /* we are filled, wait until a buffer gets popped or when we
          * flush. */
-        g_cond_wait (priv->cond, priv->mutex);
+        g_cond_wait (appsrc->priv->cond, appsrc->priv->mutex);
       } else {
         /* no need to wait for free space, we just pump more data into the
          * queue hoping that the caller reacts to the enough-data signal and
@@ -1477,10 +1249,10 @@ gst_app_src_push_buffer_full (GstAppSrc * appsrc, GstBuffer * buffer,
   GST_DEBUG_OBJECT (appsrc, "queueing buffer %p", buffer);
   if (!steal_ref)
     gst_buffer_ref (buffer);
-  g_queue_push_tail (priv->queue, buffer);
-  priv->queued_bytes += GST_BUFFER_SIZE (buffer);
-  g_cond_broadcast (priv->cond);
-  g_mutex_unlock (priv->mutex);
+  g_queue_push_tail (appsrc->priv->queue, buffer);
+  appsrc->priv->queued_bytes += GST_BUFFER_SIZE (buffer);
+  g_cond_broadcast (appsrc->priv->cond);
+  g_mutex_unlock (appsrc->priv->mutex);
 
   return GST_FLOW_OK;
 
@@ -1490,7 +1262,7 @@ flushing:
     GST_DEBUG_OBJECT (appsrc, "refuse buffer %p, we are flushing", buffer);
     if (steal_ref)
       gst_buffer_unref (buffer);
-    g_mutex_unlock (priv->mutex);
+    g_mutex_unlock (appsrc->priv->mutex);
     return GST_FLOW_WRONG_STATE;
   }
 eos:
@@ -1498,7 +1270,7 @@ eos:
     GST_DEBUG_OBJECT (appsrc, "refuse buffer %p, we are EOS", buffer);
     if (steal_ref)
       gst_buffer_unref (buffer);
-    g_mutex_unlock (priv->mutex);
+    g_mutex_unlock (appsrc->priv->mutex);
     return GST_FLOW_UNEXPECTED;
   }
 }
@@ -1511,9 +1283,6 @@ eos:
  * Adds a buffer to the queue of buffers that the appsrc element will
  * push to its source pad.  This function takes ownership of the buffer.
  *
- * When the block property is TRUE, this function can block until free
- * space becomes available in the queue.
- *
  * Returns: #GST_FLOW_OK when the buffer was successfuly queued.
  * #GST_FLOW_WRONG_STATE when @appsrc is not PAUSED or PLAYING.
  * #GST_FLOW_UNEXPECTED when EOS occured.
@@ -1549,85 +1318,30 @@ gst_app_src_push_buffer_action (GstAppSrc * appsrc, GstBuffer * buffer)
 GstFlowReturn
 gst_app_src_end_of_stream (GstAppSrc * appsrc)
 {
-  GstAppSrcPrivate *priv;
-
+  g_return_val_if_fail (appsrc, GST_FLOW_ERROR);
   g_return_val_if_fail (GST_IS_APP_SRC (appsrc), GST_FLOW_ERROR);
 
-  priv = appsrc->priv;
-
-  g_mutex_lock (priv->mutex);
+  g_mutex_lock (appsrc->priv->mutex);
   /* can't accept buffers when we are flushing. We can accept them when we are 
    * EOS although it will not do anything. */
-  if (priv->flushing)
+  if (appsrc->priv->flushing)
     goto flushing;
 
   GST_DEBUG_OBJECT (appsrc, "sending EOS");
-  priv->is_eos = TRUE;
-  g_cond_broadcast (priv->cond);
-  g_mutex_unlock (priv->mutex);
+  appsrc->priv->is_eos = TRUE;
+  g_cond_broadcast (appsrc->priv->cond);
+  g_mutex_unlock (appsrc->priv->mutex);
 
   return GST_FLOW_OK;
 
   /* ERRORS */
 flushing:
   {
-    g_mutex_unlock (priv->mutex);
     GST_DEBUG_OBJECT (appsrc, "refuse EOS, we are flushing");
     return GST_FLOW_WRONG_STATE;
   }
 }
 
-/**
- * gst_app_src_set_callbacks:
- * @appsrc: a #GstAppSrc
- * @callbacks: the callbacks
- * @user_data: a user_data argument for the callbacks
- * @notify: a destroy notify function
- *
- * Set callbacks which will be executed when data is needed, enough data has
- * been collected or when a seek should be performed.
- * This is an alternative to using the signals, it has lower overhead and is thus
- * less expensive, but also less flexible.
- *
- * If callbacks are installed, no signals will be emited for performance
- * reasons.
- *
- * Since: 0.10.23
- */
-void
-gst_app_src_set_callbacks (GstAppSrc * appsrc,
-    GstAppSrcCallbacks * callbacks, gpointer user_data, GDestroyNotify notify)
-{
-  GDestroyNotify old_notify;
-  GstAppSrcPrivate *priv;
-
-  g_return_if_fail (GST_IS_APP_SRC (appsrc));
-  g_return_if_fail (callbacks != NULL);
-
-  priv = appsrc->priv;
-
-  GST_OBJECT_LOCK (appsrc);
-  old_notify = priv->notify;
-
-  if (old_notify) {
-    gpointer old_data;
-
-    old_data = priv->user_data;
-
-    priv->user_data = NULL;
-    priv->notify = NULL;
-    GST_OBJECT_UNLOCK (appsrc);
-
-    old_notify (old_data);
-
-    GST_OBJECT_LOCK (appsrc);
-  }
-  priv->callbacks = *callbacks;
-  priv->user_data = user_data;
-  priv->notify = notify;
-  GST_OBJECT_UNLOCK (appsrc);
-}
-
 /*** GSTURIHANDLER INTERFACE *************************************************/
 
 static GstURIType
@@ -1643,7 +1357,6 @@ gst_app_src_uri_get_protocols (void)
 
   return protocols;
 }
-
 static const gchar *
 gst_app_src_uri_get_uri (GstURIHandler * handler)
 {
