@@ -22,13 +22,13 @@
 /**
  * SECTION:element-ffmpegcolorspace
  *
+ * Convert video frames between a great variety of colorspace formats.
+ *
  * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -v videotestsrc ! video/x-raw-yuv,format=\(fourcc\)YUY2 ! ffmpegcolorspace ! ximagesink
- * </programlisting>
- * </para>
+ * ]|
  * </refsect2>
  */
 
@@ -412,7 +412,8 @@ gst_ffmpegcsp_get_unit_size (GstBaseTransform * btrans, GstCaps * caps,
    * GStreamer currently puts it into the caps as 'palette_data' field,
    * so for paletted data the frame size avpicture_get_size() returns is
    * 1024 bytes larger than what GStreamer expects. */
-  if (gst_structure_has_field (structure, "palette_data")) {
+  if (gst_structure_has_field (structure, "palette_data") &&
+      ctx->pix_fmt == PIX_FMT_PAL8) {
     *size -= 4 * 256;           /* = AVPALETTE_SIZE */
   }
 
@@ -466,8 +467,7 @@ gst_ffmpegcsp_transform (GstBaseTransform * btrans, GstBuffer * inbuf,
   if (result == -1)
     goto not_supported;
 
-  /* copy timestamps */
-  gst_buffer_stamp (outbuf, inbuf);
+  /* baseclass copies timestamps */
   GST_DEBUG ("from %d -> to %d done", space->from_pixfmt, space->to_pixfmt);
 
   return GST_FLOW_OK;
