@@ -25,6 +25,8 @@
 
 #include <gst/gst.h>
 #include <gst/base/gstbasetransform.h>
+#include <gst/audio/audio.h>
+#include <gst/audio/gstaudiofilter.h>
 
 G_BEGIN_DECLS
 
@@ -42,36 +44,33 @@ G_BEGIN_DECLS
 typedef struct _GstVolume GstVolume;
 typedef struct _GstVolumeClass GstVolumeClass;
 
-typedef enum {
-  GST_VOLUME_FORMAT_NONE,
-  GST_VOLUME_FORMAT_INT, 	 
-  GST_VOLUME_FORMAT_FLOAT 	 
-} GstVolumeFormat;
-
 /**
  * GstVolume:
  *
  * Opaque data structure.
  */
 struct _GstVolume {
-  GstBaseTransform element;
+  GstAudioFilter element;
 
   void (*process)(GstVolume*, gpointer, guint);
 
   gboolean mute;
-  gint   volume_i32, real_vol_i32;
-  gint   volume_i24, real_vol_i24; /* the _i(nt) values get synchronized with the */
-  gint   volume_i16, real_vol_i16; /* the _i(nt) values get synchronized with the */
-  gint   volume_i8, real_vol_i8;   /* the _i(nt) values get synchronized with the */
-  gfloat volume_f, real_vol_f; /* _f(loat) values on each update */
+  gfloat volume;
+
+  gboolean current_mute;
+  gfloat current_volume;
+
+  gint   current_vol_i32;
+  gint   current_vol_i24; /* the _i(nt) values get synchronized with the */
+  gint   current_vol_i16; /* the _i(nt) values get synchronized with the */
+  gint   current_vol_i8;   /* the _i(nt) values get synchronized with the */
   
   GList *tracklist;
-  GstVolumeFormat format;       /* caps variables */
-  gint width;
+  gboolean negotiated;
 };
 
 struct _GstVolumeClass {
-  GstBaseTransformClass parent_class;
+  GstAudioFilterClass parent_class;
 };
 
 GType gst_volume_get_type (void);
